// Defining the json data for fish species
var treeData = [  { "name": "Mudd",    "parent": "null",    "children": [      {        "name": "Labrador Retriever/Chow Chow Mix",        "parent": "Mudd",        "children": [          {            "name": "Labrador Retriever",            "parent": "Labrador Retriever/Chow Chow Mix"          },          {            "name": "Chow Chow Mix",            "parent": "Labrador Retriever/Chow Chow Mix"          }        ]
      },
      {
        "name": "Labrador Retriever/Golden retriver/ German Shepherd Dog Cross",
        "parent": "Mudd",
        "children": [
          {
            "name": "German Shepherd Dog",
            "parent": "Labrador Retriever/Golden retriver/ German Shepherd Dog Cross",
            "children": [
              {
                "name": "German Shepherd Dog",
                "parent": "German Shepherd Dog"
              },
              {
                "name": "German Shepherd Dog",
                "parent": "German Shepherd Dog"
              },         
            ]
          },
          {
            "name": "Labrador Retriever/Golden retriver Cross",
            "parent": "Labrador Retriever/Golden retriver/ German Shepherd Dog Cross",
            "children": [
              {
                "name": "Lab Retriever US Field",
                "parent": "Labrador Retriever/Golden retriver Cross"
              },
              {
                "name": " Golden Retriever Show",
                "parent": "Labrador Retriever/Golden retriver Cross"
              }
            ]
          }
        ]
      }
    ]
  }
];


// setting the width of the screen using margin function
var margin = {top: 0, right: 120, bottom: 0, left: 200},
	width = 1160 - margin.right - margin.left,
	height = 400 - margin.top - margin.bottom;
	
var i = 0,
	duration = 750,
	root;
//defining the layout of the tree structure
var tree = d3.layout.tree()
	.size([height, width]);
//setting the external values using diagonal funtion
var diagonal = d3.svg.diagonal()
	.projection(function(d) { return [d.y, d.x]; });

//appending the attribute width and margin values
var svg = d3.select("body").append("svg")
	.attr("width", width + margin.right + margin.left)
	.attr("height", height + margin.top + margin.bottom)
  .append("g")
	.attr("transform", "translate(" + margin.left + "," + margin.top + ")");

//Setting the root values of tree
root = treeData[0];
root.x0 = height / 2;
root.y0 = 0;


//Defining the update funtion to keep root values updated
update(root);

d3.select(self.frameElement).style("height", "500px");

function update(source) {

//Creating the link between nodes
  var nodes = tree.nodes(root).reverse(),
	  links = tree.links(nodes);

 
  nodes.forEach(function(d) { d.y = d.depth * 180; });

  
  var node = svg.selectAll("g.node")
	  .data(nodes, function(d) { return d.id || (d.id = ++i); });

 
  var nodeEnter = node.enter().append("g")
	  .attr("class", "node")
	  .attr("transform", function(d) { return "translate(" + source.y0 + "," + source.x0 + ")"; })
	  .on("click", click);

  nodeEnter.append("circle")
	  .attr("r", 1e-6)
	  .style("fill", function(d) { return d._children ? "lightsteelblue" : "#fff"; });

  nodeEnter.append("text")
	  .attr("x", function(d) { return d.children || d._children ? -13 : 13; })
	  .attr("dy", ".35em")
	  .attr("text-anchor", function(d) { return d.children || d._children ? "end" : "start"; })
	  .text(function(d) { return d.name; })
	  .style("fill-opacity", 1e-6);


  var nodeUpdate = node.transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + d.y + "," + d.x + ")"; });

  nodeUpdate.select("circle")
	  .attr("r", 15)
	  .style("fill", function(d) { return d._children ? "lightsteelblue" : "teal"; });

  nodeUpdate.select("text")
	  .style("fill-opacity", 1);

// Moving the exiting nodes to new position parent
  var nodeExit = node.exit().transition()
	  .duration(duration)
	  .attr("transform", function(d) { return "translate(" + source.y + "," + source.x + ")"; })
	  .remove();

  nodeExit.select("circle")
	  .attr("r", 1e-6);

  nodeExit.select("text")
	  .style("fill-opacity", 1e-6);

 
  var link = svg.selectAll("path.link")
	  .data(links, function(d) { return d.target.id; });

 
  link.enter().insert("path", "g")
	  .attr("class", "link")
	  .attr("d", function(d) {
		var o = {x: source.x0, y: source.y0};
		return diagonal({source: o, target: o});
	  });

 		// Transition to new positions
  link.transition()
	  .duration(duration)
	  .attr("d", diagonal);


  link.exit().transition()
	  .duration(duration)
	  .attr("d", function(d) {
		var o = {x: source.x, y: source.y};
		return diagonal({source: o, target: o});
	  })
	  .remove();

  nodes.forEach(function(d) {
	d.x0 = d.x;
	d.y0 = d.y;
  });
}


function click(d) {
  if (d.children) {
	d._children = d.children;
	d.children = null;
  } else {
	d.children = d._children;
	d._children = null;
  }
  update(d);
}
